package org.openxma.dsl.core.converter;

import java.util.List;

import org.eclipse.xtext.common.services.DefaultTerminalConverters;
import org.eclipse.xtext.conversion.IValueConverter;
import org.eclipse.xtext.conversion.ValueConverter;
import org.eclipse.xtext.conversion.ValueConverterException;
import org.eclipse.xtext.conversion.impl.AbstractToStringConverter;
import org.eclipse.xtext.conversion.impl.IDValueConverter;
import org.eclipse.xtext.nodemodel.INode;
import org.eclipse.xtext.util.Strings;

import com.google.inject.Inject;

public class CoreDslConverter extends DefaultTerminalConverters {

    @Inject
    private QualifiedNameConverter qualifiedNameConverter;

    @Inject
    private QualifiedNameConverterWithWildCard qualifiedNameConverterWithWildCard;
    
    @Inject
    private SINTConverter sINTConverter;

    private final static class SINTConverter extends AbstractToStringConverter<Integer> {
        @Override
        protected Integer internalToValue(String string, INode node) throws ValueConverterException {
            return Integer.valueOf(string);
        }
    }

    private static class QualifiedNameConverter extends IDValueConverter {
        
        public String toString(String value) {
            assertValidValue(value);
            return toEscapedString(value);
        }

        protected String toEscapedString(String value) {
            List<String> segments = Strings.split(value, ".");
            StringBuilder builder = new StringBuilder();
            boolean isFirst = true;
            for (String segment : segments) {
                if (!isFirst)
                    builder.append(".");
                isFirst = false;
                if (mustEscape(segment))
                    builder.append("^");
                builder.append(segment);
            }
            return builder.toString();
        }
        
        public String toValue(String string, INode node) {
            if (string == null)
                return null;
            boolean isEscaped = string.indexOf("^") != -1;
            if (isEscaped) {
                return string.replaceAll("\\^", "");
            }
            return string;
        }
    }
    
    private static class QualifiedNameConverterWithWildCard extends QualifiedNameConverter {
        @Override
        protected boolean mustEscape(String value) {
            return !"*".equals(value) && super.mustEscape(value);
        }
    }

    @ValueConverter(rule = "SINT")
    public IValueConverter<Integer> SINT() {
        return sINTConverter;
    }

    @ValueConverter(rule = "QualifiedNameWithWildCard")
    public IValueConverter<String> QualifiedNameWithWildCard() {
        return qualifiedNameConverterWithWildCard;
    }
    
    
    @ValueConverter(rule = "QualifiedName")
    public IValueConverter<String> QualifiedName() {
        return qualifiedNameConverter;
    }
}
